package de.inso_gmbh.io;

import java.io.*;
import java.util.Date;

/**
 * Class represents entries in a tar file.
 * 
 * @author Bernd Lachner, InSo GmbH
 */
public class TarEntry
{
	public static final int BLOCK_SIZE = 512;
	public static final int NAME_SIZE = 100;
		
	public static final int MODE_SUID = 004000;
	public static final int MODE_SGID = 002000;
	public static final int MODE_SVTX = 001000;	
	public static final int MODE_UREAD = 000400;
	public static final int MODE_UWRITE = 000200;
	public static final int MODE_UEXEC = 000100;	
	public static final int MODE_GREAD = 000040;
	public static final int MODE_GWRITE = 000020;
	public static final int MODE_GEXEC = 000010;	
	public static final int MODE_OREAD = 000004;
	public static final int MODE_OWRITE = 000002;
	public static final int MODE_OEXEC = 000001;

	
	private String m_sName;
	private String m_sUserName = "";
	private String m_sGroupName = "";
	private int m_iMode = 0;
	private int m_iUserID = 0;
	private int m_iGroupID = 0;
	private long m_lSize = 0;
	private long m_lModificationTime = 0;

	private class MemoryPrintWriter extends PrintWriter
	{
		private ByteArrayOutputStream m_oByteArrayStream = null;
		
		public MemoryPrintWriter(ByteArrayOutputStream oByteArrayStream)
		{
			super(oByteArrayStream);
			m_oByteArrayStream = oByteArrayStream;
		}
		
		public ByteArrayOutputStream getByteArrayStream()
		{
			return m_oByteArrayStream;
		}
	}

	/**
	 * Sets the data size of the entry.
	 * 
	 * @param lSize Data size. 0 for directories.
	 */
	public void setSize(long lSize)
	{
		m_lSize = lSize;
	}

	/**
	 * Sets the mode flags of the entry.
	 * 
	 * @param iMode Combination of the mode flags.
	 */			
	public void setMode(int iMode)
	{
		m_iMode = iMode;
	}

	/**
	 * Sets the id of the user who owns the file.
	 * 
	 * @param iUserID User id.
	 */
	public void setUserID(int iUserID)
	{
		m_iUserID = iUserID;
	}

	/**
	 * Sets the name of the user who owns the file.
	 * 
	 * @param sUserName User name. Max. 31 charactes.
	 */
	public void setUserName(String sUserName)
	{
		m_sUserName = sUserName;
	}

	/**
	 * Sets the id of the group which owns the file.
	 * 
	 * @param iGroupID Group id.
	 */	
	public void setGroupID(int iGroupID)
	{
		m_iGroupID = iGroupID;
	}

	/**
	 * Sets the name of the group which owns the file.
	 * 
	 * @param sGroupName Group name. Max. 31 charactes.
	 */
	public void setGroupName(String sGroupName)
	{
		m_sGroupName = sGroupName;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param sName File or directory name of the entry.
	 */
	public TarEntry(String sName)
	{
		m_sName = sName;
		setModificationTime(new Date());
	}

	/**
	 * Sets the modification time of the entry.
	 * 
	 * @param lModificationTime The modification time/data as number of 
	 * seconds since January 1, 1970, 00:00:00 GMT format.
	 */
	public void setModificationTime(long lModificationTime)
	{
		m_lModificationTime = lModificationTime;
	}
		
	/**
	 * Sets the modification time of the entry.
	 * 
	 * @param oDate THe modification time/date as a Java date object.
	 */
	public void setModificationTime(Date oDate)
	{
		this.m_lModificationTime = oDate.getTime() / 1000;
	}
	
	/**
	 * Fills a databuffer until a given position with 0 bytes.
	 * 
	 * @param oPrintWriter Databuffer.
	 * @param iPos Position until the databuffer is filled with 0 bytes.
	 */
	private void FillUntilPos(MemoryPrintWriter oPrintWriter, int iPos)
	{
		// Fill with 0 bytes
		for (int i = oPrintWriter.getByteArrayStream().size(); i < iPos; i++)
			oPrintWriter.getByteArrayStream().write(0);
	}

	/**
	 * Writes a string in a databuffer and fills unused bytes with 0.
	 * 
	 * @param oPrintWriter Databuffer.
	 * @param sString String to write in the databuffer at the actual position.
	 * @param iStringLen Length of the string filed in the databuffer. Unused bytes will be filled with 0.
	 */
	private void writeString(MemoryPrintWriter oPrintWriter, String sString, int iStringLen)
	{
		int iFillPos = oPrintWriter.getByteArrayStream().size() + iStringLen;
		oPrintWriter.print(sString);
		oPrintWriter.flush();
		FillUntilPos(oPrintWriter, iFillPos);
	}
	
	/**
	 * Converts a long number to a string in octal format.
	 * 
	 * @param lOctalNum Number ro convert to  a string.
	 * @param iDigits Number of octal digits in the string.
	 * @return String with the number in octal format. Unused digits will be filled with 0.
	 * */
	private String getOctalString(long lOctalNum, int iDigits)
	{
		String sOctalNum = Long.toOctalString(lOctalNum);
		for (int i = sOctalNum.length(); i < iDigits; i++)
		{
			sOctalNum = "0" + sOctalNum;			
		}
		return sOctalNum;
	}

	/**
	 * Generate the header of the entry.
	 * 
	 * @param lChecksum Checksum of the header. If the checksum is 0, the checksum filed in the header will be filled with spaces.
	 * @return The 512 byte header of the tar entry.
	 */			
	private byte[] getHeader(long lChecksum)
	{
		MemoryPrintWriter oHeaderWriter = new MemoryPrintWriter(new ByteArrayOutputStream(BLOCK_SIZE));	
		// Write name of the entry 
		writeString(oHeaderWriter, m_sName, NAME_SIZE);
		// Write mode
		writeString(oHeaderWriter, getOctalString(m_iMode, 7), 8);
		// Write user and group ID
		writeString(oHeaderWriter, getOctalString(m_iUserID, 7), 8);
		writeString(oHeaderWriter, getOctalString(m_iGroupID, 7), 8);	
		// Write file size
		writeString(oHeaderWriter, getOctalString(m_lSize, 11), 12);	
		// Write modification time
		writeString(oHeaderWriter, getOctalString(m_lModificationTime, 11), 12);	
		// Write checksum
		if (lChecksum != 0)
		{
			writeString(oHeaderWriter, getOctalString(lChecksum, 6), 7);
		}
		else
		{
			writeString(oHeaderWriter, "       ", 7);
		}
		writeString(oHeaderWriter, " ", 1);
		// Write file type
		if (m_lSize == 0)
		{
			writeString(oHeaderWriter, "5", 1);
		}
		else
		{
			writeString(oHeaderWriter, "0", 1);
		}
		FillUntilPos(oHeaderWriter, 257);
		// Write format and version of tar format
		writeString(oHeaderWriter, "ustar  ", 8);
		// Write user name
		writeString(oHeaderWriter, m_sUserName, 32);
		// Write group name
		writeString(oHeaderWriter, m_sGroupName, 32);
		// Fill rest of the Header with 0 bytes
		FillUntilPos(oHeaderWriter, BLOCK_SIZE);
		return oHeaderWriter.getByteArrayStream().toByteArray();
	}

	/**
	 * Calcs the checksum of the header.
	 * 
	 * @return Checksum of the header.
	 */		
	private long calcChecksum()
	{
		// Get header with checksum set to 0
		byte abHeader[] = getHeader(0);
		// Calc checksum
		long lCheckSum = 0;
		for (int i = 0; i < abHeader.length; i++)
		{
			lCheckSum += abHeader[i];
		}
		return lCheckSum;
	}

	/**
	 * Generate the header of the entry.
	 * 
	 * @return The 512 byte header of the tar entry.
	 */	
	public byte[] getHeader()
	{
		return getHeader(calcChecksum());
	}
}
