package de.inso_gmbh.ipkexportplugin.wizards;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;

/**
 * The base class for all ipk wizard pages.
 */
public abstract class ExtWizardPage extends WizardPage 
{
	protected static final char m_acAllowedUnixFileChar[] 
	= { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.', '-', '_' };
	
	 
	/**
	 * Constructor.
	 * 
	 * @param sPageName The name of the page.
	 */
	protected ExtWizardPage(String sPageName)
	{
		super(sPageName);
	}
	
	/**
	 * Constructor.
	 * 
	 * @param sPageName The name of the page.
	 * @param sTitle The title for this wizard page, or null if none.
	 * @param oTitleImage The image descriptor for the title of this wizard page, or null if none.
	 */
	protected ExtWizardPage(String sPageName, String sTitle, ImageDescriptor oTitleImage)
	{
		super(sPageName, sTitle, oTitleImage);
	}
	
	/**
	 * Creates a new composite widget with a grid layout.
	 * 
	 * @param oParent The parent widget to insert the new grid container.
	 * @param iColumns Number of columns of the grid.
	 * @return Composite widget with the grid layout.
	 */
	protected Composite makeGridContainer(Composite oParent, int iColumns)
	{
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		Composite oContainer = new Composite(oParent, SWT.NULL);
		oContainer.setLayoutData(oGridData);
		GridLayout oLayout = new GridLayout();
		oContainer.setLayout(oLayout);
		oLayout.numColumns = iColumns;
		return oContainer;
	}
	
	/**
	 * Adds a label with a following text widget to a container with a grid layout.
	 *  
	 * @param oContainer Containert to add the widgets.
	 * @param sLabelText The text for the label widget.
	 * @param iColumns The number of columns the text widget should occupy.
	 * @param bReadOnly If true, the text widget is read only.
	 * @return The new text widget.
	 */
	protected Text addLabelAndText(Composite oContainer, String sLabelText, int iColumns, boolean bReadOnly)
	{
		Label oLabel = new Label(oContainer, SWT.NULL);
		oLabel.setText(sLabelText);
		Text oText = null;
		if (bReadOnly == true)
		{
			oText = new Text(oContainer, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY);
		}
		else
		{
			oText = new Text(oContainer, SWT.BORDER | SWT.SINGLE);
			oText.addModifyListener(new ModifyListener() 
			{
				public void modifyText(ModifyEvent oModifyEvent) 
				{
					dialogChanged();
				}
			});
		}
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		oGridData.horizontalSpan = iColumns;
		oText.setLayoutData(oGridData);
		return oText;
	}
	
	/**
	 * Checks if the text in a text widget is empty (no characters or only spaces).
	 * 
	 * @param oText The text widget to check.
	 * @return True if empty, otherwise false.
	 */
	protected boolean isTextEmpty(Text oText)
	{
		if (oText.getText().length() > 0)
		{
			String sText = oText.getText();
			for (int i = 0; i < sText.length(); i++)
			{
				if (sText.charAt(i) != ' ')
					return false;
			}
		}
		return true;
	}
	
	/**
	 * Adds a label with a following combo widget to a container with a grid layout.
	 * 
	 * @param oContainer Container to add the widgets.
	 * @param sLabelText The text for the label widget.
	 * @param asComboEntries String array with the combo box entries.
	 * @return The new combo widget.
	 */
	protected Combo addLabelAndCombo(Composite oContainer, String sLabelText, String asComboEntries[])
	{
		Label oLabel = new Label(oContainer, SWT.NULL);
		oLabel.setText(sLabelText);
		Combo oCombo = new Combo(oContainer, SWT.READ_ONLY | SWT.BORDER | SWT.SINGLE);
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		oCombo.setLayoutData(oGridData);
		for (int i = 0; i < asComboEntries.length; i++)
		{
			oCombo.add(asComboEntries[i]);
		}
		oCombo.select(0);
		oCombo.addModifyListener(new ModifyListener() 
		{
			public void modifyText(ModifyEvent oModifyEvent) 
			{
				dialogChanged();
			}
		});
		return oCombo;
	}
	
	/**
	 * Gets a string value from a string array related to the combo box index.
	 * 
	 * @param oCombo The combo box to get the index.
	 * @param asComboValues The string array to get the value.
	 * @return The value related to the combo box index or a empty string if no value in the string array for the index.
	 */	
	protected String getComboValue(Combo oCombo, String asComboValues[])
	{
		if (oCombo.getSelectionIndex() < 0 || oCombo.getSelectionIndex() >= asComboValues.length)
		{
			return "";
		}
		else
		{
			return asComboValues[oCombo.getSelectionIndex()];
		}
	}

	/**
	 * A verify handle method for a text widget. It verifies if the text of the widget  
	 * contains allowed filename.
	 * 
	 * This verify handle method only allows the following charcters:
	 * 0..9, a..z, A..Z, ., _, -
	 * 
	 * Actual file systems on Linux, windows, etc. allows more characters, but this should 
	 * be a secure methode to check if a filename is valid on most systems.
	 * 
	 * @param oVerifyEvent Verify event.
	 */
	protected void verifyFileName(VerifyEvent oVerifyEvent) 
	{
		if (oVerifyEvent.text.length() > 0)
		{
			// oVerifyEvent.character doesn't work with the SWT Linux GTK version it is always empty
			// therefore oVerifyEvent.text is used
			char cVerify = Character.toLowerCase(oVerifyEvent.text.charAt(0));
			if (Character.isISOControl(cVerify))
			{
				oVerifyEvent.doit = true;
				return;
			}
			for (int i = 0; i < m_acAllowedUnixFileChar.length; i++)
			{
				if (cVerify == m_acAllowedUnixFileChar[i])
				{
					oVerifyEvent.doit = true;
					return;
				}
			}
			oVerifyEvent.doit = false;			
		}
		else
		{
			oVerifyEvent.doit = true;
		}
	}

	/**
	 * Sets the status of the wizard page.
	 *  
	 * @param sErrorMessage Error message or null if no error. If null the wizard page is marked as completed. 
	 */
	protected void updateStatus(String sErrorMessage) 
	{
		setErrorMessage(sErrorMessage);
		setPageComplete(sErrorMessage == null);
	}

	/**
	 * Method must be implemented from subclasses. 
	 * Method is calle if a combo box or text box value is changed.
	 */
	protected abstract void dialogChanged();
}
