package de.inso_gmbh.ipkexportplugin.wizards;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.zip.GZIPOutputStream;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.ui.jarpackager.JarPackageData;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.IWorkbench;

import de.inso_gmbh.io.TarEntry;
import de.inso_gmbh.io.TarOutputStream;


/**
 * This is a wizard for exporting Java programs in a ipk package.
 */
public class IpkExportWizard extends Wizard implements IExportWizard 
{
	private IpkExportWizardPageJarSettings m_oJarSettingsPage;
	private IpkExportWizardPageControlSettings m_oControlSettingsPage;
	private IpkExportWizardPageDesktopSettings m_oDesktopSettingsPage;

	private IStructuredSelection m_oWorkbenchSelection;

	private static final int TAR_STD_FLAGS = TarEntry.MODE_OREAD | TarEntry.MODE_GREAD | TarEntry.MODE_UREAD | TarEntry.MODE_UWRITE;
	private static final int TAR_DIR_FLAGS = TarEntry.MODE_OREAD | TarEntry.MODE_OEXEC | TarEntry.MODE_GREAD | TarEntry.MODE_GEXEC
	| TarEntry.MODE_UREAD | TarEntry.MODE_UWRITE | TarEntry.MODE_UEXEC;
	private static final int TAR_EXEC_FLAGS = TarEntry.MODE_OREAD | TarEntry.MODE_OEXEC | TarEntry.MODE_GREAD | TarEntry.MODE_GEXEC
	| TarEntry.MODE_UREAD | TarEntry.MODE_UWRITE | TarEntry.MODE_UEXEC;

	/**
	 * Constructor for IpkgExportWizard.
	 */
	public IpkExportWizard() 
	{
		super();
		setNeedsProgressMonitor(true);
	}
	
	/**
	 * Adding the settings pages to the wizard.
	 */
	public void addPages() 
	{
		m_oJarSettingsPage = new IpkExportWizardPageJarSettings(m_oWorkbenchSelection, this);
		addPage(m_oJarSettingsPage);
		m_oControlSettingsPage = new IpkExportWizardPageControlSettings(this);
		addPage(m_oControlSettingsPage);
		m_oDesktopSettingsPage = new IpkExportWizardPageDesktopSettings(this);
		addPage(m_oDesktopSettingsPage);
	}

	/**
	 * Checks if a file with the given filename exists.
	 * 
	 * @param sFileName Filename including path which is to check.
	 * @return true if filename exist, otherwise false.
	 */
	static private boolean fileExists(String sFileName)
	{
		try 
		{
			FileInputStream oTestStream = new FileInputStream(sFileName);
			oTestStream.close();
		}
		catch (Exception oException) 
		{
			return false;
		}
		return true;
	}
	
	/**
	 * Generates a temporary file name including the path.
	 * 
	 * @param sBaseName The basename of the file (first part of the filename).
	 * @param sExtension The extension of the file (last part of the filename).
	 * @return Filename for creating a temporary file.
	 */
	static private String getTempFileName(String sBaseName, String sExtension)
	{
		String sTempDir = System.getProperty("java.io.tmpdir", "");
		if (sTempDir.endsWith(File.separator) == false)
			sTempDir += File.separator;
		for (long i = 0; i < Long.MAX_VALUE; i++)
		{
			String sFileName = sTempDir + sBaseName +  new Long(i).toString() + sExtension;
			if (fileExists(sFileName) == false)
				return sFileName;
		}
		return null;
	}
	
	/**
	 * Creates a new tar entry for a tar output stream.
	 * 
	 * @param sEntryName The (file)name of the tar entry.
	 * @param iMode The file modes for the tar entry.
	 * @return The created tar entry.
	 */	
	static private TarEntry createTarEntry(String sEntryName, int iMode)
	{
		TarEntry oNewEntry = new TarEntry(sEntryName);
		oNewEntry.setMode(iMode);
		oNewEntry.setUserID(0);
		oNewEntry.setUserName("root");
		oNewEntry.setGroupID(0);
		oNewEntry.setGroupName("root");
		return oNewEntry;
	}

	/**
	 * Checks if a string is empty (no characters or only spaces).
	 * 
	 * @param sString String for checking.
	 * @return true if string is empty, otherwise false.
	 */
	static private boolean isStringEmpty(String sString)
	{
		if (sString.length() > 0)
		{
			for (int i = 0; i < sString.length(); i++)
			{
				if (sString.charAt(i) != ' ')
					return false;
			}
		}
		return true;	
	}
	
	/**
	 * Writes a string to a writer object if the value is not empty (no characters or only spaces).
	 *  
	 * @param oWriter Writer wher the string should be written.
	 * @param sKey The key (first part of the resulting string) which sould be written to the writer object.
	 * @param sValue The value (second part of the resulting string) which sould be written to the writer object.
	 * @param bLineFeed True to add linefeed, othwerwise false. 
	 */
	static private void writeIfNotEmpty(PrintWriter oWriter, String sKey, String sValue, boolean bLineFeed)	
	{
		if (isStringEmpty(sValue) == false)
		{
			oWriter.write(sKey + sValue);
			if (bLineFeed)
				oWriter.write("\n");						
		}
	}
	
	/**
	 * Copies a file to the actual entry of a tar output stream.
	 * 
	 * @param oTarStream The tar output stream where the file should copied to.
	 * @param sFileName The name (including path) of the source file which should be copied to the tar output stream.
	 * @throws FileNotFoundException
	 * @throws IOException
	 */
	static private void copyFileToTarEntry(TarOutputStream oTarStream, String sFileName) throws FileNotFoundException, IOException
	{
		FileInputStream oInputStream = new FileInputStream(sFileName);
		int iCopyData;
   		while ((iCopyData = oInputStream.read ()) != -1)
	       oTarStream.write(iCopyData);	
		oInputStream.close();
	}
	
	/**
	 * Writes the ipk package as described on the wizard pages.
	 * 
	 * @param sTempJarFileName The name of the existing jar file which should be included in the package.
	 */
	private void writeIpk(String sTempJarFileName)
	{
		try
		{
			TarOutputStream oIpkStream = new TarOutputStream(new GZIPOutputStream(new FileOutputStream(this.m_oControlSettingsPage.getIpkFileName())));						

			// Write control file			
			oIpkStream.putNextEntry(createTarEntry("./", TAR_DIR_FLAGS));
			oIpkStream.closeEntry();
			oIpkStream.putNextEntry(createTarEntry("./control.tar.gz", TAR_STD_FLAGS));
			GZIPOutputStream oControlGZIPStream = new GZIPOutputStream(oIpkStream);			
			TarOutputStream oControlTarStream = new TarOutputStream(oControlGZIPStream);	
			oControlTarStream.putNextEntry(createTarEntry("./", TAR_DIR_FLAGS));
			oControlTarStream.closeEntry();
			oControlTarStream.putNextEntry(createTarEntry("./control", TAR_STD_FLAGS));
			PrintWriter oControlWriter = new PrintWriter(oControlTarStream);
			writeIfNotEmpty(oControlWriter, "Package: ", m_oControlSettingsPage.getPackageName(), true);
			writeIfNotEmpty(oControlWriter, "Version: ", m_oControlSettingsPage.getPackageVersion(), true);
			writeIfNotEmpty(oControlWriter, "Description: ", m_oControlSettingsPage.getPackageDescription(), true);
			writeIfNotEmpty(oControlWriter, "Maintainer: ", m_oControlSettingsPage.getPackageMaintainer(), true);
			writeIfNotEmpty(oControlWriter, "Depends: ", m_oControlSettingsPage.getPackageDepends(), true);
			writeIfNotEmpty(oControlWriter, "Architecture: ", m_oControlSettingsPage.getPackageArchitecture(), true);
			writeIfNotEmpty(oControlWriter, "Priority: ", m_oControlSettingsPage.getPackagePriority(), true);
			writeIfNotEmpty(oControlWriter, "Section: ", m_oControlSettingsPage.getPackageSection(), true);			
			oControlWriter.flush();
			oControlTarStream.closeEntry();	
			oControlTarStream.closeArchive();			
			oControlGZIPStream.finish();
			oIpkStream.closeEntry();			

			// Write data file			
			oIpkStream.putNextEntry(createTarEntry("./", TAR_DIR_FLAGS));
			oIpkStream.closeEntry();
			oIpkStream.putNextEntry(createTarEntry("./data.tar.gz", TAR_STD_FLAGS));
			GZIPOutputStream oDataGZIPStream = new GZIPOutputStream(oIpkStream);			
			TarOutputStream oDataTarStream = new TarOutputStream(oDataGZIPStream);	
			oDataTarStream.putNextEntry(createTarEntry("opt/", TAR_DIR_FLAGS));
			oDataTarStream.closeEntry();
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/", TAR_DIR_FLAGS));
			oDataTarStream.closeEntry();
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/apps/", TAR_DIR_FLAGS));
			oDataTarStream.closeEntry();
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/apps/"+m_oDesktopSettingsPage.getTypeDir()+"/", TAR_DIR_FLAGS));
			oDataTarStream.closeEntry();			
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/apps/"+m_oDesktopSettingsPage.getTypeDir()+"/"+this.m_oDesktopSettingsPage.getDesktopFileName(), TAR_STD_FLAGS));
			PrintWriter oDataWriter = new PrintWriter(oDataTarStream);
			oDataWriter.write("[Desktop Entry]\n");
			writeIfNotEmpty(oDataWriter, "Comment=", m_oDesktopSettingsPage.getComment(), true);
			writeIfNotEmpty(oDataWriter, "Exec=", m_oDesktopSettingsPage.getExecName(), true);			
			writeIfNotEmpty(oDataWriter, "Icon=", m_oDesktopSettingsPage.getIconFileName(), true);	
			writeIfNotEmpty(oDataWriter,"Type=", m_oDesktopSettingsPage.getType(), true);		
			writeIfNotEmpty(oDataWriter, "Name=", m_oDesktopSettingsPage.getApplicationName(), true);
			oDataWriter.flush();			
			oDataTarStream.closeEntry();			
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/bin/", TAR_DIR_FLAGS));
			oDataTarStream.closeEntry();
			// Write start script
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/bin/"+m_oDesktopSettingsPage.getExecName(), TAR_EXEC_FLAGS));
			oDataWriter.write("#!/bin/bash\n");
			oDataWriter.write("evm");
			writeIfNotEmpty(oDataWriter, " ", this.m_oDesktopSettingsPage.getJvmOptions(), false);
			oDataWriter.write(" -XappName=" + this.m_oDesktopSettingsPage.getExecName());
			oDataWriter.write(" -cp /opt/QtPalmtop/bin/"+this.m_oDesktopSettingsPage.getJarName() + " " + m_oJarSettingsPage.getMainClass());
			for (int i = 0; i < m_oDesktopSettingsPage.getMainArgumentsNum(); i++)
			{
				oDataWriter.write(" $"+Integer.toString(i+1));
			}			
			oDataWriter.flush();
			oDataTarStream.closeEntry();
			// Copy icon file
			if (isStringEmpty(m_oDesktopSettingsPage.getIconFileName()) == false)
			{
				oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/pics/"+m_oDesktopSettingsPage.getIconFileName(), TAR_STD_FLAGS));			
				copyFileToTarEntry(oDataTarStream, m_oDesktopSettingsPage.getSourceIconFileName());
				oDataTarStream.closeEntry();
			}
			// Copy jar to data file						
			oDataTarStream.putNextEntry(createTarEntry("opt/QtPalmtop/bin/"+this.m_oDesktopSettingsPage.getJarName(), TAR_STD_FLAGS));			
			copyFileToTarEntry(oDataTarStream, sTempJarFileName);
			oDataTarStream.closeEntry();
			oDataTarStream.closeArchive();			
			oDataGZIPStream.finish();		       
			oIpkStream.closeEntry();
			
			// Write debian file
			oIpkStream.putNextEntry(createTarEntry("./", TAR_DIR_FLAGS));
			oIpkStream.closeEntry();			
			oIpkStream.putNextEntry(createTarEntry("./debian-binary", TAR_STD_FLAGS));
			PrintWriter oIpkWriter = new PrintWriter(oIpkStream);
			oIpkWriter.write("2.0\n");
			oIpkWriter.flush();
			oIpkStream.closeEntry();
			
			oIpkStream.close();
		}
		catch (FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
		finally
		{
			// Delete temporary jar file			
			File oFile = new File(sTempJarFileName);
			oFile.delete();
		}
	}
	
	/**
	 * This method is called when 'Finish' button is pressed in
	 * the wizard. 
	 */
	public boolean performFinish() 
	{
		// Check if ipk file exists
		File oCheckFile = new File(this.m_oControlSettingsPage.getIpkFileName());
		if (oCheckFile.isFile() == true)
		{
			// Ipk file exist. Ask user if it can be overwritten.
			MessageBox oMessageBox = new MessageBox(getShell(), SWT.ICON_QUESTION | SWT.YES | SWT.NO);
			oMessageBox.setText("Export ipk package");
			oMessageBox.setMessage("The ipk file " + this.m_oControlSettingsPage.getIpkFileName() + " already exsits! Overwrite?");
			if (oMessageBox.open() == SWT.NO)
			{
				// Don't overwrite exiting ipk file
				return false;
			}
		}
		final String sTempJarFileName = getTempFileName(m_oDesktopSettingsPage.getExecName(),".tmp");
		IRunnableWithProgress op = new IRunnableWithProgress() 
		{
			public void run(IProgressMonitor monitor) throws InvocationTargetException 
			{
				try 
				{
					doFinish(sTempJarFileName, monitor);
				} 
				catch (CoreException e) 
				{
					throw new InvocationTargetException(e);
				} 
				finally 
				{
					monitor.done();
				}
			}
		};
		try 
		{
			getContainer().run(true, false, op);
			writeIpk(sTempJarFileName);
		} 
		catch (InterruptedException e) 
		{
			return false;
		} 
		catch (InvocationTargetException e) 
		{
			Throwable realException = e.getTargetException();
			MessageDialog.openError(getShell(), "Error", realException.getMessage());
			return false;
		}
		return true;
	}
	
	
	/**
	 * The worker method. It will create the jar file.
	 */
	private void doFinish(String sTempJarFileName, IProgressMonitor monitor) throws CoreException 
	{
		try
		{
			JarPackageData oJarPackageData = this.m_oJarSettingsPage.getJarPackageData();
			oJarPackageData.setJarLocation(new Path(sTempJarFileName));
			oJarPackageData.setCompress(true);
			oJarPackageData.createJarExportRunnable(getShell()).run(monitor);
		}		
		catch (InvocationTargetException e)
		{
			e.printStackTrace();
		}
		catch (InterruptedException e)
		{
			System.out.println(e.getMessage());
		}
		catch (RuntimeException e)
		{
			e.printStackTrace();
		}
	}
	
	/**
	 * Generates a excetion for the wizard.
	 * 
	 * @param sErrorMessage The error message of the exception.
	 * @throws CoreException
	 */
	private void throwCoreException(String sErrorMessage) throws CoreException 
	{
		IStatus status = new Status(IStatus.ERROR, "de.insogmbh.ipkgplugin", IStatus.OK, sErrorMessage, null);
		throw new CoreException(status);
	}

	/**
	 * We will accept the selection in the workbench to see if
	 * we can initialize from it.
	 * @see IWorkbenchWizard#init(IWorkbench, IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) 
	{
		this.m_oWorkbenchSelection = selection;
	}
}