package de.inso_gmbh.ipkexportplugin.wizards;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;

import de.inso_gmbh.ipkexportplugin.IpkExportPlugin;

import java.io.*;

/**
 * The "control settings" wizard page allows setting the parameters of the ipk control file.
 */
public class IpkExportWizardPageControlSettings extends ExtWizardPage 
{
	private Text m_oPackageNameText = null; 
	private Text m_oPackageVersionText = null; 
	private Text m_oPackageDescriptionText = null;
	private Text m_oPackageMaintainerText = null;
	private Text m_oPackageDependsText = null;
	private Combo m_oPriorityCombo = null;
	private Combo m_oSectionCombo = null;
	private Combo m_oArchitectureCombo = null;
	private Text m_oIpkName = null;
	private Text m_oIpkDirText = null;
	
	private static final String m_asPriority[] = {"required", "standard", "important", "optional", "extra"};
	private static final String m_asPriorityCombo[] = {"required", "standard", "important", "optional", "extra"};
	private static final String m_asSection[] = { "admin", "base", "comm", "editors", "extras", "graphics", "libs", "misc", "net", "text", "web", "x11"}; 			
	private static final String m_asSectionCombo[] = { "admin", "base", "comm", "editors", "extras", "graphics", "libs", "misc", "net", "text", "web", "x11"}; 			
	private static final String m_asArchitecture[] = { "all", "arm"}; 			
	private static final String m_asArchitectureCombo[] = { "all", "arm"}; 			
	
	private boolean m_bIsCreated = false;
	
	private IpkExportWizard m_oParent = null;
	
	/**
	 * Constructor for "control settings" wizard page.
	 * 
	 * @param oParent Reference to the parent wizard object.
	 */
	public IpkExportWizardPageControlSettings(IpkExportWizard oParent) 
	{
		super("ControlSettingsPage", "IPK Export / Control settings", IpkExportPlugin.createImageDescriptor("icons/ipk_export_wizard.gif"));
		setDescription("The parameters of the control file for the IPK package.");
		this.m_oParent = oParent;
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) 
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Pacakge description:");
		this.createControlSettingsSelectionGroup(container);
		label = new Label(container, SWT.NULL);		
		label.setText("Select the export destination:");
		this.createIPKSelectionGroup(container);
		this.m_bIsCreated = true;

		dialogChanged();
		setControl(container);
	}
	
	/**
	 * Creates the widgets for the control file parameters. 
	 * 
	 * @param oParent Parent widget.
	 */
	private void createControlSettingsSelectionGroup(Composite oParent)
	{
		Composite container = makeGridContainer(oParent, 2);
		
		this.m_oPackageNameText = addLabelAndText(container, "Package name:", 1, false);
		this.m_oPackageNameText.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyFileName(oVerifyEvent);
			}
		});
		this.m_oPackageVersionText = addLabelAndText(container, "Package version:", 1, false);
		this.m_oPackageVersionText.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyFileName(oVerifyEvent);
			}
		});
		this.m_oPackageDescriptionText = addLabelAndText(container, "Package description:", 1, false);
		this.m_oPackageMaintainerText = addLabelAndText(container, "Package maintainer:", 1, false);
		this.m_oPackageDependsText = addLabelAndText(container, "Package depends on:", 1, false);
		this.m_oArchitectureCombo = addLabelAndCombo(container, "Architecture:", IpkExportWizardPageControlSettings.m_asArchitectureCombo);
		this.m_oPriorityCombo = addLabelAndCombo(container, "Priority:", IpkExportWizardPageControlSettings.m_asPriorityCombo); 
		this.m_oPriorityCombo.select(3);
		this.m_oSectionCombo = addLabelAndCombo(container, "Section:", IpkExportWizardPageControlSettings.m_asSectionCombo);
		this.m_oSectionCombo.select(4);
		this.m_oIpkName = addLabelAndText(container, "Ipk name:", 1, true);
	}
	
	/**
	 * Creates the widgets for the ipk destination directory selection.
	 *
	 * @param oParent Parent widget.
	 */
	private void createIPKSelectionGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 3);

		this.m_oIpkDirText = addLabelAndText(oContainer, "IPK dir:", 1, false);
		this.m_oIpkDirText.setText(System.getProperty("user.home"));
		Button button = new Button(oContainer, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() 
		{
			public void widgetSelected(SelectionEvent e) 
			{
				handleIpkDirSelection();
			}
		});
	}

	/**
	 * Gets the name parameter for the control file.
	 * 
	 * @return Name.
	 */
	public String getPackageName()
	{
		return this.m_oPackageNameText.getText();
	}	
	
	/**
	 * Gets the version parameter for the control file.
	 * 
	 * @return Version.
	 */
	public String getPackageVersion()
	{
		return this.m_oPackageVersionText.getText();
	}	

	/**
	 * Gets the description parameter for the control file.
	 * 
	 * @return Description.
	 */
	public String getPackageDescription()
	{
		return this.m_oPackageDescriptionText.getText();
	}
	
	/**
	 * Gets the maintainer parameter for the control file.
	 * 
	 * @return Maintainer.
	 */
	public String getPackageMaintainer()
	{
		return this.m_oPackageMaintainerText.getText();
	}	

	/**
	 * Gets the depends parameter for the control file.
	 * 
	 * @return Depends.
	 */
	public String getPackageDepends()
	{
		return this.m_oPackageDependsText.getText();
	}

	/**
	 * Gets the architecture parameter for the control file.
	 * 
	 * @return Architecture.
	 */
	public String getPackageArchitecture()
	{
		return getComboValue(this.m_oArchitectureCombo, IpkExportWizardPageControlSettings.m_asArchitecture);
	}
		
	/**
	 * Gets the priority parameter for the control file.
	 * 
	 * @return Priotrity.
	 */
	public String getPackagePriority()
	{
		return getComboValue(this.m_oPriorityCombo, IpkExportWizardPageControlSettings.m_asPriority);
	}

	/**
	 * Gets the section parameter for the control file.
	 * 
	 * @return Section.
	 */
	public String getPackageSection()
	{
		return getComboValue(this.m_oSectionCombo, IpkExportWizardPageControlSettings.m_asSection);
	}

	/**
	 * Gets the name of the ipk package which sould be generated. The name is build as follows:
	 * 
	 * Name_Version_Architecture.ipk
	 * 
	 * @return Name of the ipk package.
	 */
	public String getIpkName()
	{
		return this.m_oIpkName.getText();
	}
		
	/**
	 * Gets the name of the ipk package including the full destination path.
	 * 
	 * @return Name of the ipk package including destination path.
	 */
	public String getIpkFileName()
	{
		String sDir = this.m_oIpkDirText.getText().trim();
		if (sDir.endsWith(File.separator) == false)
			sDir += File.separator; 
		return (sDir + this.m_oIpkName.getText());
	}
	

	/**
	 * Handle methode for the ipk destination dir browse button.
	 */
	private void handleIpkDirSelection() 
	{
		DirectoryDialog oSaveDirDialog = new DirectoryDialog(getShell());
		oSaveDirDialog.setMessage("Choose IPK file destination directory");
		String sSaveDirName = oSaveDirDialog.open();
		if (sSaveDirName != null)
		{
			this.m_oIpkDirText.setText(sSaveDirName);
		}
	}

	/**
	 * Is called when the dialog settings have changed.
	 */	
	protected void dialogChanged() 
	{
		if (m_bIsCreated == false)
			return;

		// Generate Ipk name
		this.m_oIpkName.setText((getPackageName() + "_" + getPackageVersion() + "_" + getPackageArchitecture() + ".ipk").toLowerCase());

		// Check if all required fields are set			
		if (isTextEmpty(this.m_oPackageNameText) || isTextEmpty(this.m_oPackageVersionText) ||
			isTextEmpty(this.m_oPackageMaintainerText) || isTextEmpty(this.m_oPackageDescriptionText))
		{
			updateStatus("Package, Version, Architecture, Maintainer, and Description are required!");
			return;
		}
		
		// Check if ipk destination directory is a valid directory
		File oCheckDir = new File(this.m_oIpkDirText.getText());
		if (oCheckDir.isDirectory() == false)
		{
			updateStatus("Ipk destination directory is not valid!");
			return;
		}
		updateStatus(null);
	}
}