package de.inso_gmbh.ipkexportplugin.wizards;

import org.eclipse.jface.dialogs.*;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.core.resources.*;
import org.eclipse.swt.events.*;
import org.eclipse.ui.dialogs.*;
import org.eclipse.jdt.core.*;
import org.eclipse.jdt.core.search.*;
import org.eclipse.jdt.ui.*;
import org.eclipse.jdt.ui.jarpackager.*;

import de.inso_gmbh.ipkexportplugin.IpkExportPlugin;

import java.util.*;

/**
 * The "jar settings" wizard page allows setting the jar parameters for
 * the jar file included in the ipk package.
 */
public class IpkExportWizardPageJarSettings extends ExtWizardPage implements ICheckStateListener, ISelectionChangedListener
{
	// Widgets
	private Text m_oMainClassText = null; 
	private Button m_oMainClassBrowseButton = null;
	private CheckboxTreeViewer m_oPackageViewer = null;
	protected CheckboxTableViewer m_oResourceViewer = null;
		
	// Object for jar generation	
	private JarPackageData m_oJarPackageData = new JarPackageData();
	
	// Map with selected resources key = selected object in package viewer, value = list of selected objects in ressource viewer
	private HashMap m_oSelectedTableElements = new HashMap();
	// Actual selected object in package viewer
	private Object m_oSelectedPackageElement = null;
	
	private IStructuredSelection m_oWorkbenchSelection;
	private IpkExportWizard m_oParent = null;

	/**
	 * Constructor for "jar settings" wizard page.
	 * 
	 * @param oWorkbenchSelection Workbench selection from eclipse.
	 * @param oParent Reference to the parent wizard object.
	 */	
	public IpkExportWizardPageJarSettings(IStructuredSelection oWorkbenchSelection, IpkExportWizard oParent) 
	{
		super("JarSettingsPage", "IPK Export / Jar settings", IpkExportPlugin.createImageDescriptor("icons/ipk_export_wizard.gif"));
		setDescription("The settings for the Jar file which will be included in the IPK package.");
		this.m_oWorkbenchSelection = oWorkbenchSelection;
		this.m_oParent = oParent;
	}

	private IJavaProject getSelectedProject()
	{
		Iterator it = m_oWorkbenchSelection.iterator();
		while (it.hasNext())
		{
			Object oObject = it.next();
			if (oObject instanceof IJavaElement)
			{
				IJavaElement oJavaElement = (IJavaElement) oObject;
				IJavaProject oJavaProject = oJavaElement.getJavaProject();
				if (oJavaProject != null)
				{
					return oJavaProject;
				}				
			}
		}
		return null;
	}

	/**
	 * Is called when a Item in the package or the resource viewer is checked or unchecked.
	 * 
	 * @param oEvent Check state changed event.
	 */
	public void checkStateChanged(CheckStateChangedEvent oEvent) 
	{
		if (oEvent.getCheckable().equals(this.m_oResourceViewer))
		{
			// Resource viewer (table) element checked or unchecked 
			checkResourceElement(oEvent.getElement(), oEvent.getChecked());
		}
		else
		{
			// Package viewer (tree) element checked or unchecked
			checkPackageElement(oEvent.getElement(), oEvent.getChecked());
		}
		setPackageState();
		dialogChanged();
	}
	
	/**
	 * Add a item which is checked in the resource viewer to the list of selected items.
	 * 
	 * @param oResourceElement The checked item.
	 * @param bChecked True if checked, False if unchecked.
	 */
	private void checkResourceElement(Object oResourceElement, boolean bChecked)
	{
		// Get list of selected resources of the package the element which should be checked or unchecked belongs
		// or create a new list if it don't exist.
		ArrayList listSelected = (ArrayList) this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement);
		if (listSelected == null)
		{
			listSelected = new ArrayList();
		}
		// Add / remove the element wich should be checked / unchecked from the list
		if (bChecked == true)
		{
			if (listSelected.contains(oResourceElement) == false)
			{
				listSelected.add(oResourceElement);
			}
		}
		else
		{
			if (listSelected.contains(oResourceElement) == true)
			{
				listSelected.remove(oResourceElement);
			}
		}
		// If already a item is in the list add the list to map of selected items, otherwise remove the list from the map
		if (listSelected.size() > 0)
		{	
			this.m_oSelectedTableElements.put(this.m_oSelectedPackageElement, listSelected);
		}
		else
		{
			this.m_oSelectedTableElements.remove(this.m_oSelectedPackageElement);
		}
	}
	
	/**
	 * Add a item which is checked in the package viewer and all its childrens to the list of selected items.
	 * 
	 * @param oPackageElement The checked item.
	 * @param bChecked True if checked, False if unchecked.
	 */
	private void checkPackageElement(Object oPackageElement, boolean bChecked)
	{
		// If the package content is viewed in the resource viewer check all items in the resource viewer
		if (this.m_oSelectedPackageElement == oPackageElement)
			this.m_oResourceViewer.setAllChecked(bChecked);
		// Check or uncheck all the content of the package (resources)
		if (bChecked == true)
		{
			ArrayList listSelected = getResourceElements(oPackageElement);
			this.m_oSelectedTableElements.put(oPackageElement, listSelected);			
		}
		else
		{
			this.m_oSelectedTableElements.remove(oPackageElement);
		}
		// Recursivly check or uncheck all childrens
		ArrayList listChildrens = getPackageChildren(oPackageElement);
		Iterator it = listChildrens.iterator();
		while (it.hasNext())
		{
			checkPackageElement(it.next(), bChecked);
		}
	}
	
	/**
	 * Set the check state of the actually selected items in the package viewer.
	 * 
	 * Package which content is only partially selected will be grayed checked,
	 * packages which content is completely selected will be white checked and all
	 * other packages will be unchecked..
	 */
	private void setPackageState()
	{
		// List of all checked package (tree) elments	
		ArrayList listCheckedPackageElements = new ArrayList();
		// Add all selected packages and its parent packages to the list of checked elements
		Iterator it = m_oSelectedTableElements.keySet().iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			listCheckedPackageElements.add(oPackageElement);
			while ((oPackageElement = ((ITreeContentProvider)this.m_oPackageViewer.getContentProvider()).getParent(oPackageElement)) != null)
			{
				listCheckedPackageElements.add(oPackageElement);
			}			
		}
		// List of all grayed package (tree) elements
		ArrayList listGrayedPackageElements = new ArrayList();
		// Add all checked package elements which resources are only partially checked to the list of grayed elements
		it = listCheckedPackageElements.iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			int iSelectedElements = 0;
			ArrayList listSelected = (ArrayList)this.m_oSelectedTableElements.get(oPackageElement);
			if (listSelected != null)
			{
				iSelectedElements = listSelected.size();
			}			
			if (this.getResourceElements(oPackageElement).size() != iSelectedElements)
			{
				listGrayedPackageElements.add(oPackageElement);
			}
		}
		// Add all checked package elements which childs are only partially checked to the list of grayed elements
		it = listCheckedPackageElements.iterator();
		while (it.hasNext())
		{
			Object oPackageElement = it.next();
			if (listGrayedPackageElements.contains(oPackageElement) == true)
			{
				continue;
			}
			ArrayList listChilds = getPackageChildren(oPackageElement);
			Iterator jt	= listChilds.iterator();
			while (jt.hasNext())
			{
				Object oChild = jt.next();
				if (listCheckedPackageElements.contains(oChild) == false || listGrayedPackageElements.contains(oChild) == true)
				{
					listGrayedPackageElements.add(oPackageElement);
					break;
				}
			}
		}

		this.m_oPackageViewer.setCheckedElements(listCheckedPackageElements.toArray());		
		this.m_oPackageViewer.setGrayedElements(listGrayedPackageElements.toArray());
	}
	
	/**
	 * 
	 * @author Bernd Lachner, InSo GmbH
	 *
	 * Filter class to filter packages for the pacakge viewer.
	 */
	private class PackageViewerFilter extends ViewerFilter
	{
		public boolean select(Viewer oViewer, Object oParentElement, Object oElement) 
		{
			if (oElement instanceof IJavaElement)
			{
				// Filter out non Java Project, Model or Package elements
				IJavaElement JavaElement = (IJavaElement) oElement;
				int iType = JavaElement.getElementType();
				if (iType != IJavaElement.JAVA_MODEL && iType != IJavaElement.JAVA_PROJECT &&
					iType != IJavaElement.PACKAGE_FRAGMENT_ROOT && iType != IJavaElement.PACKAGE_FRAGMENT)
				{
					return false;
				}
				// Filter out empty packages
				if (oElement instanceof IPackageFragment)
				{
					IPackageFragment PackageFragment = (IPackageFragment) oElement;
					try
					{
						return PackageFragment.hasChildren() || (PackageFragment.getNonJavaResources().length > 0);
					}
					catch (JavaModelException e)
					{
						return false;
					}					
				}
				// Filter out Jar files
				if (oElement instanceof IPackageFragmentRoot)
				{
					IPackageFragmentRoot PackageFragmentRoot = (IPackageFragmentRoot) oElement;	
					if (PackageFragmentRoot.isArchive())
						return false;					
				}
				return true;
			}
			return false;
		}
	}
	
	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) 
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Select the resources to export:");
		this.createPackageSelectionGroup(container);
		label = new Label(container, SWT.NULL);
		label.setText("Select application entry point:");
		this.createMainClassSelectionGroup(container);		

		initialize();
		dialogChanged();
		setControl(container);
	}

	/**
	 * Is called when the selection in the package viewer (tree) changed.
	 *  
	 * @param oEvent Selection changed event.
	 */
	public void selectionChanged(SelectionChangedEvent oEvent)	
	{
		this.m_oSelectedPackageElement = ((IStructuredSelection)oEvent.getSelection()).getFirstElement();
		if (this.m_oSelectedPackageElement != null)
		{
			this.m_oResourceViewer.setInput(this.m_oSelectedPackageElement);
			// Set previous checked elements if necessary
			if (this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement) != null)
			{
				this.m_oResourceViewer.setCheckedElements(((ArrayList)this.m_oSelectedTableElements.get(this.m_oSelectedPackageElement)).toArray());
			}
		}
	}
	
	/**
	 * Creates the widgets for the package / resource selection.
	 * 
	 * @param oParent Parent widget.
	 */
	protected void createPackageSelectionGroup(Composite oParent)
	{
		GridData gd = new GridData(GridData.FILL_HORIZONTAL);
		Composite container = new Composite(oParent, SWT.NULL);
		container.setLayoutData(gd);
		GridLayout layout = new GridLayout(2, true);
		container.setLayout(layout);

		ITreeContentProvider treeContentProvider = new StandardJavaElementContentProvider()		
		{
			public boolean hasChildren(Object element) 
			{
				// no + should shown in front of packages
				return !(element instanceof IPackageFragment) && super.hasChildren(element);
			}
		};		
		Tree tree = new Tree(container, SWT.CHECK | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);	
		gd.heightHint = 160;
		tree.setLayoutData(gd);
		m_oPackageViewer = new CheckboxTreeViewer(tree);
		m_oPackageViewer.setContentProvider(treeContentProvider);
		m_oPackageViewer.setSorter(new JavaElementSorter());
		m_oPackageViewer.setLabelProvider(new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_BASICS | JavaElementLabelProvider.SHOW_OVERLAY_ICONS | JavaElementLabelProvider.SHOW_SMALL_ICONS));
		m_oPackageViewer.setInput(JavaCore.create(ResourcesPlugin.getWorkspace().getRoot()));//this.m_oParent.getSelectedProject());		
		m_oPackageViewer.addFilter(new PackageViewerFilter());
		m_oPackageViewer.addCheckStateListener(this);
		m_oPackageViewer.addSelectionChangedListener(this);
				
		IStructuredContentProvider tableContentProvider = new StandardJavaElementContentProvider()
		{
			public Object[] getElements(Object oParent)
			{
				Object aoElements[] = super.getElements(oParent);
				int iElementNum = 0;
				for (int i = 0; i < aoElements.length; i++)
				{
					if (aoElements[i] instanceof ICompilationUnit || aoElements[i] instanceof IFile)
						iElementNum++;
				}
				Object aoFilteredElements[] = new Object[iElementNum];
				iElementNum = 0;
				for (int i = 0; i < aoElements.length; i++)
				{
					if (aoElements[i] instanceof ICompilationUnit || aoElements[i] instanceof IFile)
					{
						aoFilteredElements[iElementNum] = aoElements[i];
						iElementNum++;
					}
				}
				
				return aoFilteredElements;
			}
		};
		Table table = new Table(container, SWT.CHECK | SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);	
		gd.heightHint = 160;
		table.setLayoutData(gd);
		m_oResourceViewer = new CheckboxTableViewer(table);		
		m_oResourceViewer.setContentProvider(tableContentProvider);//new StandardJavaElementContentProvider());
		m_oResourceViewer.setLabelProvider(new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_BASICS | JavaElementLabelProvider.SHOW_OVERLAY_ICONS | JavaElementLabelProvider.SHOW_SMALL_ICONS));
		m_oResourceViewer.addCheckStateListener(this);
	}
	
	/**
	 * Creates the widgets for the main class selection.
	 * 
	 * @param oParent PArent widget.
	 */
	protected void createMainClassSelectionGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 3);
		
		m_oMainClassText = addLabelAndText(oContainer, "Main class:", 1, true);
		m_oMainClassBrowseButton = new Button(oContainer, SWT.PUSH);
		m_oMainClassBrowseButton.setText("Browse...");
		m_oMainClassBrowseButton.setEnabled(false);
		m_oMainClassBrowseButton.addSelectionListener(new SelectionAdapter() 
		{
			public void widgetSelected(SelectionEvent e) 
			{
				handleMainClassSelection();
			}
		});
	}

	/**
	 * Handle methode for the main class selection button.
	 */
	private void handleMainClassSelection()
	{
		try
		{
			ProgressMonitorDialog oProgressMonitorDialog = new ProgressMonitorDialog(getShell());
			SelectionDialog oSelectMainClassDialog = 
			JavaUI.createMainTypeDialog(getShell(), oProgressMonitorDialog, getJavaSearchScope(), 0, false);
			oSelectMainClassDialog.setMessage("Select main class");
			oSelectMainClassDialog.open();
			Object[] oResult = oSelectMainClassDialog.getResult();
			if (oResult != null)
			{
				IType oMainClass = (IType) oResult[0];
				this.m_oJarPackageData.setManifestMainClass(oMainClass);
				m_oMainClassText.setText(oMainClass.getFullyQualifiedName());
				dialogChanged();				
			}
		}
		catch (Throwable e)
		{
			e.printStackTrace();
		}
	}	

	/**
	 * Gets all selected Java elements.
	 * 
	 * @return Selected Java elements.
	 */	
	public IJavaElement[] getSelectedJavaElements()
	{
		int iJavaElementNum = 0;
		ArrayList listSelected = getSelectedElements();
		Iterator it = listSelected.iterator();
		while (it.hasNext())
		{
			if (it.next() instanceof IJavaElement)
			{
				iJavaElementNum++;
			}
		}
		IJavaElement aoJavaElements[] = new IJavaElement[iJavaElementNum];
		iJavaElementNum = 0;
		it = listSelected.iterator();
		while (it.hasNext())
		{
			Object oJavaElement = it.next();
			if (oJavaElement instanceof IJavaElement)
			{
				aoJavaElements[iJavaElementNum] = (IJavaElement) oJavaElement;
				iJavaElementNum++;
			}
		}		
		return aoJavaElements;
	}

	/**
	 * Creates a Java search scope for searching over all selected Java elements.
	 * 
	 * @return Java search scope.
	 */	
	public IJavaSearchScope getJavaSearchScope()
	{
		return SearchEngine.createJavaSearchScope(getSelectedJavaElements());
	}
	
	/**
	 * Gets all selected elements.
	 * 
	 * @return Selected elements.
	 */
	public ArrayList getSelectedElements()
	{
		ArrayList listSelected = new ArrayList();		
		Iterator it = this.m_oSelectedTableElements.values().iterator();
		while (it.hasNext())
		{
			listSelected.addAll((Collection)it.next());
		}
		return listSelected;
	}
		
	/**
	 * Tests if the current workbench selection is a suitable
	 * container to use.
	 */
	private void initialize() 
	{
		if (m_oWorkbenchSelection!=null && m_oWorkbenchSelection.isEmpty()==false && m_oWorkbenchSelection instanceof IStructuredSelection) 
		{
			IStructuredSelection ssel = (IStructuredSelection)m_oWorkbenchSelection;
			if (ssel.size()>1) return;
			Object obj = ssel.getFirstElement();
			if (obj instanceof IResource) 
			{
				IContainer container;
				if (obj instanceof IContainer)
					container = (IContainer)obj;
				else
					container = ((IResource)obj).getParent();
			}
		}
	}

	/**
	 * Gets the full name of the selected main class.
	 * @return
	 */
	public String getMainClass()
	{
		return this.m_oMainClassText.getText();
	}
	
	/**
	 * Is called when the dialog settings have changed.
	*/
	protected void dialogChanged() 
	{
		if (getSelectedJavaElements().length == 0)
		{
			updateStatus("No Java classes selected!");
			this.m_oMainClassBrowseButton.setEnabled(false);
			return;
		}
		this.m_oMainClassBrowseButton.setEnabled(true);
		JarPackageData oJarPackageData = getJarPackageData();
		if (oJarPackageData.getManifestMainClass() == null || oJarPackageData.isMainClassValid(getContainer()) == false)
		{
			updateStatus("Main class must be specified!");
			return;
		}	
		updateStatus(null);
	}

	/**
	 * Filters a elements array with a set of viewer filters.
	 * 
	 * @param aoFilters Set of viewer filters. If null, the outgoing elements are equal the incoming elements.
	 * @param aoElements Incoming elements to filter.
	 * @return Outgoing, filtered, elements.
	 */
	private ArrayList filter(ViewerFilter[] aoFilters, Object[] aoElements) 
	{
		if (aoFilters != null) 
		{
			ArrayList listFiltered = new ArrayList(aoElements.length);
			for (int i = 0; i < aoElements.length; i++) 
			{
				boolean bAdd = true;
				for (int j = 0; j < aoFilters.length; j++) 
				{
					bAdd = aoFilters[j].select(null, null, aoElements[i]);
					if (!bAdd)
						break;
				}
				if (bAdd)
					listFiltered.add(aoElements[i]);
			}
			return listFiltered;
		}
		else
		{
			// No filter, return all objects.
			ArrayList listElements = new ArrayList(aoElements.length);
			for (int i = 0; i < aoElements.length; i++)
			{
				listElements.add(aoElements[i]);
			}	
			return listElements;
		}
	}

	/**
	 * Gets the children of a package element.
	 * 
	 * @param oElement Package element which childrens should returned.
	 * @return Childrens.
	 */
	private ArrayList getPackageChildren(Object oPackageElement) 
	{
		return filter(this.m_oPackageViewer.getFilters(), ((ITreeContentProvider)this.m_oPackageViewer.getContentProvider()).getChildren(oPackageElement));
	}

	/**
	 * Gets the resources of a package element.
	 * 
	 * @param oElement Package element which resources should returned.
	 * @return Resources.
	 */
	private ArrayList getResourceElements(Object oPackageElement) 
	{
		return  filter(this.m_oResourceViewer.getFilters(), ((IStructuredContentProvider)this.m_oResourceViewer.getContentProvider()).getElements(oPackageElement));
	}
	
	/**
	 * Get the jar package data to create a jar package as described with this wizard page.
	 * 
	 * @return Jar package data.
	 */
	public JarPackageData getJarPackageData()
	{
		m_oJarPackageData.setElements(getSelectedElements().toArray());			
		return m_oJarPackageData;
	}
}

